@@ -12,6 +12,7 @@
 #               02/20/06 - P. Harvey Fixed bug where seek past end of file could
 #                          generate "substr outside string" warning
 #               06/10/06 - P. Harvey Decreased $CHUNK_SIZE from 64k to 8k
+#               11/23/06 - P. Harvey Limit reads to < 0x80000000 bytes
 #
 # Notes:        Calls the normal file i/o routines unless SeekTest() fails, in
 #               which case the file is buffered in memory to allow random access.
@@ -20,7 +21,7 @@
 #
 #               May also be used for string i/o (just pass a scalar reference)
 #
-# Legal:        Copyright (c) 2004-2006 Phil Harvey (phil at owl.phy.queensu.ca)
+# Legal:        Copyright (c) 2003-2008 Phil Harvey (phil at owl.phy.queensu.ca)
 #               This library is free software; you can redistribute it and/or
 #               modify it under the same terms as Perl itself.
 #------------------------------------------------------------------------------
@@ -32,7 +33,7 @@ require 5.002;
 require Exporter;
 
 use vars qw($VERSION @ISA @EXPORT_OK);
-$VERSION = '1.05';
+$VERSION = '1.07';
 @ISA = qw(Exporter);
 
 # constants
@@ -133,19 +134,18 @@ sub Seek($$;$)
     $whence = 0 unless defined $whence;
     my $rtnVal;
     if ($self->{TESTED} < 0) {
-        $rtnVal = 1;
+        my $newPos;
         if ($whence == 0) {
-            $self->{POS} = $num;
+            $newPos = $num;                 # from start of file
         } elsif ($whence == 1) {
-            $self->{POS} += $num;
+            $newPos = $num + $self->{POS};  # relative to current position
         } else {
-            $self->Slurp();     # read whole file into buffer
-            if ($self->{LEN} >= $num) {
-                # position relative to end of file
-                $self->{POS} = $self->{LEN} + $num;
-            } else {
-                $rtnVal = 0;    # seek error
-            }
+            $self->Slurp();                 # read whole file into buffer
+            $newPos = $num + $self->{LEN};  # relative to end of file
+        }
+        if ($newPos >= 0) {
+            $self->{POS} = $newPos;
+            $rtnVal = 1;
         }
     } else {
         $rtnVal = seek($self->{FILE_PT}, $num, $whence);
@@ -164,6 +164,9 @@ sub Read($$$)
     my $len = $_[1];
     my $rtnVal;
 
+    # avoid dying with "Negative length" error
+    return 0 if $len & 0x80000000;
+
     if ($self->{TESTED} < 0) {
         my $buff;
         my $newPos = $self->{POS} + $len;
