@@ -6,8 +6,9 @@
 # Revisions:    11/25/2003  - P. Harvey Created
 #
 # References:   1) http://park2.wakwak.com/~tsuruzoh/Computer/Digicams/exif-e.html
-#               2) Christian Koller private communication (tests with the 20D)
-#               3) Rainer Honle private communication (tests with the 5D)
+#               2) Christian Koller private communication (20D)
+#               3) Rainer Honle private communication (5D)
+#               4) David Pitcher private communication (1DmkIII firmware upgrade)
 #------------------------------------------------------------------------------
 
 package Image::ExifTool::CanonCustom;
@@ -18,7 +19,7 @@ use Image::ExifTool qw(:DataAccess);
 use Image::ExifTool::Canon;
 use Image::ExifTool::Exif;
 
-$VERSION = '1.17';
+$VERSION = '1.20';
 
 sub ProcessCanonCustom($$$);
 sub ProcessCanonCustom2($$$);
@@ -1201,7 +1202,7 @@ my %convPFn = ( PrintConv => \&ConvertPfn, PrintConvInv => \&ConvertPfnInv );
     0x0101 => [
         {
             Name => 'ExposureLevelIncrements',
-            Condition => '$$self{CameraModel} =~ /\b40D\b/',
+            Condition => '$$self{Model} =~ /\b40D\b/',
             Notes => '40D',
             PrintConv => {
                 0 => '1/3 Stop',
@@ -1228,7 +1229,7 @@ my %convPFn = ( PrintConv => \&ConvertPfn, PrintConvInv => \&ConvertPfnInv );
     0x0103 => [
         {
             Name => 'ISOExpansion',
-            Condition => '$$self{CameraModel} =~ /\b40D\b/',
+            Condition => '$$self{Model} =~ /\b40D\b/',
             Notes => '40D',
             PrintConv => {
                 0 => 'Off',
@@ -1392,7 +1393,7 @@ my %convPFn = ( PrintConv => \&ConvertPfn, PrintConvInv => \&ConvertPfnInv );
     0x010f => [
         {
             Name => 'FlashSyncSpeedAv',
-            Condition => '$$self{CameraModel} =~ /\b40D\b/',
+            Condition => '$$self{Model} =~ /\b40D\b/',
             Notes => '40D',
             PrintConv => {
                 0 => 'Auto',
@@ -1425,6 +1426,10 @@ my %convPFn = ( PrintConv => \&ConvertPfn, PrintConvInv => \&ConvertPfnInv );
         Name => 'HighlightTonePriority',
         PrintConv => \%disableEnable
     },
+    0x0204 => {
+        Name => 'AutoLightingOptimizer',
+        PrintConv => \%enableDisable,
+    },
     #### 2b) Flash exposure
     0x0304 => {
         Name => 'ETTLII',
@@ -1544,6 +1549,9 @@ my %convPFn = ( PrintConv => \&ConvertPfn, PrintConvInv => \&ConvertPfnInv );
             0 => '19 points',
             1 => 'Inner 9 points',
             2 => 'Outer 9 points',
+            3 => '19 Points, Multi-controller selectable', #4
+            4 => 'Inner 9 Points, Multi-controller selectable', #4
+            5 => 'Outer 9 Points, Multi-controller selectable', #4
         },
     },
     0x050a => {
@@ -1655,12 +1663,14 @@ my %convPFn = ( PrintConv => \&ConvertPfn, PrintConvInv => \&ConvertPfnInv );
             0 => 'Exposure comp/Aperture',
             1 => 'AF point selection',
             2 => 'ISO speed',
+            3 => 'AF point selection swapped with Exposure comp', #4
+            4 => 'ISO speed swapped with Exposure comp', #4
         },
     },
     0x0704 => [
         {
             Name => 'SetButtonWhenShooting',
-            Condition => '$$self{CameraModel} =~ /\b40D\b/',
+            Condition => '$$self{Model} =~ /\b40D\b/',
             Notes => '40D',
             PrintConv => {
                 0 => 'Normal (disabled)',
@@ -1729,7 +1739,7 @@ my %convPFn = ( PrintConv => \&ConvertPfn, PrintConvInv => \&ConvertPfnInv );
     0x080b => [
         {
             Name => 'FocusingScreen',
-            Condition => '$$self{CameraModel} =~ /\b40D\b/',
+            Condition => '$$self{Model} =~ /\b40D\b/',
             Notes => '40D',
             PrintConv => {
                 0 => 'Ef-A',
@@ -1753,8 +1763,8 @@ my %convPFn = ( PrintConv => \&ConvertPfn, PrintConvInv => \&ConvertPfnInv );
         Count => 4,
         PrintConv => [
             \%disableEnable,
-            '"6 sec: $val"',
-            '"16 sec: $val"',
+            '"6 s: $val"',
+            '"16 s: $val"',
             '"After release: $val"',
         ],
         PrintConvInv => [
@@ -1791,6 +1801,13 @@ my %convPFn = ( PrintConv => \&ConvertPfn, PrintConvInv => \&ConvertPfnInv );
             1 => 'Enable (simulates exposure)',
         },
     },
+    0x0811 => {
+        Name => 'LCDDisplayAtPowerOn',
+        PrintConv => {
+            0 => 'Display',
+            1 => 'Retain power off status',
+        },
+    },
 );
 
 #------------------------------------------------------------------------------
@@ -1935,7 +1952,7 @@ sub ProcessCanonCustom($$$)
 
     # first entry in array must be the size
     my $len = Get16u($dataPt,$offset);
-    unless ($len == $size or ($$exifTool{CameraModel}=~/\bD60\b/ and $len+2 == $size)) {
+    unless ($len == $size or ($$exifTool{Model}=~/\bD60\b/ and $len+2 == $size)) {
         $exifTool->Warn("Invalid CanonCustom data");
         return 0;
     }
@@ -1984,7 +2001,7 @@ sub WriteCanonCustom($$$)
 
     # first entry in array must be the size
     my $len = Get16u($dataPt, 0);
-    unless ($len == $dirLen or ($$exifTool{CameraModel}=~/\bD60\b/ and $len+2 == $dirLen)) {
+    unless ($len == $dirLen or ($$exifTool{Model}=~/\bD60\b/ and $len+2 == $dirLen)) {
         $exifTool->Warn("Invalid CanonCustom data");
         return undef;
     }
@@ -2032,7 +2049,7 @@ Image::ExifTool to read this information.
 
 =head1 AUTHOR
 
-Copyright 2003-2007, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2008, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
@@ -2048,7 +2065,8 @@ under the same terms as Perl itself.
 =head1 ACKNOWLEDGEMENTS
 
 Thanks to Christian Koller for his work in decoding the 20D custom
-functions, and Rainer Honle for decoding the 5D custom functions.
+functions, Rainer Honle for decoding the 5D custom functions and David
+Pitcher for adding a few undocumented 1DmkIII settings.
 
 =head1 SEE ALSO
 
