@@ -12,7 +12,7 @@ use strict;
 use vars qw($VERSION);
 use Image::ExifTool::Exif;
 
-$VERSION = '1.09';
+$VERSION = '1.20';
 
 my %coordConv = (
     ValueConv    => 'Image::ExifTool::GPS::ToDegrees($val)',
@@ -30,7 +30,8 @@ my %coordConv = (
     NOTES => q{
         When adding GPS information to an image, it is important to set all of the
         following tags: GPSLatitude, GPSLatitudeRef, GPSLongitude, GPSLongitudeRef,
-        GPSAltitude and GPSAltitudeRef.
+        GPSAltitude and GPSAltitudeRef.  ExifTool will write the required
+        GPSVersionID tag automatically if new a GPS IFD is added to an image.
     },
     0x0000 => {
         Name => 'GPSVersionID',
@@ -80,8 +81,8 @@ my %coordConv = (
     0x0006 => {
         Name => 'GPSAltitude',
         Writable => 'rational64u',
-        PrintConv => '"$val metres"',
-        PrintConvInv => '$val=~s/\s*m.*//;$val',
+        PrintConv => '$val eq "inf" ? $val : "$val m"',
+        PrintConvInv => '$val=~s/\s*m$//;$val',
     },
     0x0007 => {
         Name => 'GPSTimeStamp',
@@ -89,8 +90,31 @@ my %coordConv = (
         Writable => 'rational64u',
         Count => 3,
         Shift => 'Time',
-        ValueConv => 'Image::ExifTool::Exif::ExifTime($val)',
+        ValueConv => sub {
+            my $val = shift;
+            my ($h,$m,$s) = split ' ', $val;
+            my $f = (($h || 0) * 60 + ($m || 0)) * 60 + ($s || 0);
+            $h = int($f / 3600); $f -= $h * 3600;
+            $m = int($f / 60);   $f -= $m * 60;
+            $s = int($f);        $f -= $s;
+            $f = int($f * 1000000 + 0.5);
+            if ($f) {
+                ($f = sprintf(".%.6d", $f)) =~ s/0+$//;
+            } else {
+                $f = ''
+            }
+            return sprintf("%.2d:%.2d:%.2d$f",$h,$m,$s);
+        },
         ValueConvInv => '$val=~tr/:/ /;$val',
+        # pull time out of any format date/time string
+        # (eventually handle timezones? -- timestamp should be UTC)
+        PrintConvInv => sub {
+            my $v = shift;
+            $v =~ s/[-+].*//s; # remove timezone
+            my @a = ($v =~ /((?=\d|\.\d)\d*(?:\.\d*)?)/g);
+            push @a, '00' while @a < 3;
+            return "$a[-3]:$a[-2]:$a[-1]";
+        },
     },
     0x0008 => {
         Name => 'GPSSatellites',
@@ -101,11 +125,12 @@ my %coordConv = (
         Writable => 'string',
         Count => 2,
         PrintConv => {
-            A => 'Measurement In Progress',
-            V => 'Measurement Interoperability',
+            A => 'Measurement Active', # Exif2.2 "Measurement in progress"
+            V => 'Measurement Void',   # Exif2.2 "Measurement Interoperability" (WTF?)
+            # (meaning for 'V' taken from status code in NMEA GLL and RMC sentences)
         },
     },
-    0x000A => {
+    0x000a => {
         Name => 'GPSMeasureMode',
         Writable => 'string',
         Count => 2,
@@ -114,12 +139,12 @@ my %coordConv = (
             3 => '3-Dimensional Measurement',
         },
     },
-    0x000B => {
+    0x000b => {
         Name => 'GPSDOP',
-        Description => 'GPS Degree Of Precision',
+        Description => 'GPS Dilution Of Precision',
         Writable => 'rational64u',
     },
-    0x000C => {
+    0x000c => {
         Name => 'GPSSpeedRef',
         Writable => 'string',
         Count => 2,
@@ -129,11 +154,11 @@ my %coordConv = (
             N => 'knots',
         },
     },
-    0x000D => {
+    0x000d => {
         Name => 'GPSSpeed',
         Writable => 'rational64u',
     },
-    0x000E => {
+    0x000e => {
         Name => 'GPSTrackRef',
         Writable => 'string',
         Count => 2,
@@ -142,7 +167,7 @@ my %coordConv = (
             T => 'True North',
         },
     },
-    0x000F => {
+    0x000f => {
         Name => 'GPSTrack',
         Writable => 'rational64u',
     },
@@ -216,32 +241,35 @@ my %coordConv = (
             N => 'Nautical Miles',
         },
     },
-    0x001A => {
+    0x001a => {
         Name => 'GPSDestDistance',
         Writable => 'rational64u',
     },
-    0x001B => {
+    0x001b => {
         Name => 'GPSProcessingMethod',
         Writable => 'undef',
-        PrintConv => 'Image::ExifTool::Exif::ConvertExifText($self,$val)',
-        PrintConvInv => '"ASCII\0\0\0$val"',
+        RawConv => 'Image::ExifTool::Exif::ConvertExifText($self,$val)',
+        RawConvInv => 'Image::ExifTool::Exif::EncodeExifText($self,$val)',
     },
-    0x001C => {
+    0x001c => {
         Name => 'GPSAreaInformation',
         Writable => 'undef',
-        PrintConv => 'Image::ExifTool::Exif::ConvertExifText($self,$val)',
-        PrintConvInv => '"ASCII\0\0\0$val"',
+        RawConv => 'Image::ExifTool::Exif::ConvertExifText($self,$val)',
+        RawConvInv => 'Image::ExifTool::Exif::EncodeExifText($self,$val)',
     },
-    0x001D => {
+    0x001d => {
         Name => 'GPSDateStamp',
         Groups => { 2 => 'Time' },
         Writable => 'string',
+        Notes => 'YYYY:MM:DD',
         Count => 11,
         Shift => 'Time',
         ValueConv => 'Image::ExifTool::Exif::ExifDate($val)',
         ValueConvInv => '$val',
+        # pull date out of any format date/time string
+        PrintConvInv => '$val=~/(\d{4}).*?(\d{2}).*?(\d{2})/ ? "$1:$2:$3" : $val',
     },
-    0x001E => {
+    0x001e => {
         Name => 'GPSDifferential',
         Writable => 'int16u',
         PrintConv => {
@@ -257,40 +285,32 @@ my %coordConv = (
         Description => 'GPS Date/Time',
         Groups => { 2 => 'Time' },
         Require => {
-            0 => 'GPSDateStamp',
-            1 => 'GPSTimeStamp',
+            0 => 'GPS:GPSDateStamp',
+            1 => 'GPS:GPSTimeStamp',
         },
         ValueConv => '"$val[0] $val[1]"',
         PrintConv => '$self->ConvertDateTime($val)',
     },
-    GPSPosition => {
+    GPSLatitude => {
         Require => {
-            0 => 'GPSLatitude',
-            1 => 'GPSLongitude',
+            0 => 'GPS:GPSLatitude',
+            1 => 'GPS:GPSLatitudeRef',
         },
-        Desire => {
-            2 => 'GPSLatitudeRef',
-            3 => 'GPSLongitudeRef',
-        },
-        ValueConv => q{
-            foreach (0..1) {
-                # set sign from lat/long reference direction if it exists
-                next unless $val[$_+2];
-                $val[$_] = abs($val[$_]);
-                $val[$_] = -$val[$_] if $val[$_+2] =~ /^(S|W)/i;
-            }
-            return "$val[0] $val[1]";
-        },
-        PrintConv => q{
-            my $lat  = Image::ExifTool::GPS::ToDMS($self, $val[0], 1, 'N');
-            my $long = Image::ExifTool::GPS::ToDMS($self, $val[1], 1, 'E');
-            return "$lat, $long";
+        ValueConv => '$val[1] =~ /^S/i ? -$val[0] : $val[0]',
+        PrintConv => 'Image::ExifTool::GPS::ToDMS($self, $val, 1, "N")',
+    },
+    GPSLongitude => {
+        Require => {
+            0 => 'GPS:GPSLongitude',
+            1 => 'GPS:GPSLongitudeRef',
         },
+        ValueConv => '$val[1] =~ /^W/i ? -$val[0] : $val[0]',
+        PrintConv => 'Image::ExifTool::GPS::ToDMS($self, $val, 1, "E")',
     },
 );
 
 # add our composite tags
-Image::ExifTool::AddCompositeTags(\%Image::ExifTool::GPS::Composite);
+Image::ExifTool::AddCompositeTags('Image::ExifTool::GPS');
 
 #------------------------------------------------------------------------------
 # Convert degrees to DMS, or whatever the current settings are
@@ -316,7 +336,7 @@ sub ToDMS($$;$$)
         if ($doPrintConv eq '1') {
             $fmt = ($exifTool->Options('CoordFormat') || q{%d deg %d' %.2f"}) . $ref;
         } else {
-            $fmt = "%d,%.2f$ref";   # use XMP standard format
+            $fmt = "%d,%.6f$ref";   # use XMP standard format
         }
         # count the number of format specifiers
         $num = ($fmt =~ tr/%/%/);
@@ -348,7 +368,7 @@ sub ToDegrees($;$)
     my ($d, $m, $s) = ($val =~ /((?:[+-]?)(?=\d|\.\d)\d*(?:\.\d*)?)/g);
     my $deg = ($d || 0) + (($m || 0) + ($s || 0)/60) / 60;
     # make negative if S or W coordinate
-    $deg = -$deg if $doSign and $val =~ /[^A-Z](S|W)$/i;
+    $deg = -$deg if $doSign ? $val =~ /[^A-Z](S|W)$/i : $deg < 0;
     return $deg;
 }
 
@@ -372,7 +392,7 @@ GPS (Global Positioning System) meta information in EXIF data.
 
 =head1 AUTHOR
 
-Copyright 2003-2006, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2008, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
