@@ -10,10 +10,11 @@ package Image::ExifTool::HtmlDump;
 
 use strict;
 use vars qw($VERSION);
+use Image::ExifTool;    # only for FinishTiffDump()
 
-$VERSION = '1.07';
+$VERSION = '1.21';
 
-sub DumpTable($$$;$$$$);
+sub DumpTable($$$;$$$$$);
 sub Open($$$;@);
 sub Write($@);
 
@@ -21,7 +22,7 @@ my ($bkgStart, $bkgEnd, $bkgSpan);
 
 my $htmlHeader1 = <<_END_PART_1_;
 <!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN"
-"http://www.w3.org/TR/1998/REC-html40-19980424/loose.dtd">
+ "http://www.w3.org/TR/1998/REC-html40-19980424/loose.dtd">
 <html>
 <head>
 <title>
@@ -33,47 +34,45 @@ my $htmlHeader2 = <<_END_PART_2_;
 <style type="text/css">
 <!--
 /* character style ID's */
-#D { color: #000000 } /* default color */
-#V { color: #ff0000 } /* duplicate block 1 */
-#W { color: #004400 } /* normal block 1 */
-#X { color: #ff4488 } /* duplicate block 2 */
-#Y { color: #448844 } /* normal block 2 */
-#U { color: #cc8844 } /* unused data block */
-#H { color: #0000ff } /* highlighted tag name */
-#M { text-decoration: underline } /* maker notes data */
+.D { color: #000000 } /* default color */
+.V { color: #ff0000 } /* duplicate block 1 */
+.W { color: #004400 } /* normal block 1 */
+.X { color: #ff4488 } /* duplicate block 2 */
+.Y { color: #448844 } /* normal block 2 */
+.U { color: #cc8844 } /* unused data block */
+.H { color: #0000ff } /* highlighted tag name */
+.F { color: #aa00dd } /* actual offset differs */
+.M { text-decoration: underline } /* maker notes data */
 /* table styles */
 table.dump {
-  border-top-style: solid;
-  border-bottom-style: solid;
-  border-width: 1;
-  border-color: #888888;
-  padding-top: 4;
-  padding-bottom: 4;
-}
-table.dump td {
-  padding-left: 4;
-  padding-right: 4;
+  border-top: 1px solid gray;
+  border-bottom: 1px solid gray;
 }
+table.dump td { padding: .2em .3em }
 td.c2 {
-  border-left-style: solid;
-  border-right-style: solid;
-  border-width: 1;
-  border-color: #888888;
-  padding-left: 4;
-  padding-right: 4;
+  border-left: 1px solid gray;
+  border-right: 1px solid gray;
 }
+pre   { margin: 0 }
+table { font-size: .9em }
+body  { color: black; background: white }
 -->
 </style>
 <script language="JavaScript" type="text/JavaScript">
 <!-- Begin
 var t = new Array;
-function GetSpansByClass(classname) {
+var mspan = new Array;
+function GetElementsByClass(classname, tagname) {
   var found = new Array();
-  var list = document.getElementsByTagName('span');
+  var list = document.getElementsByTagName(tagname);
   var len = list.length;
   for (var i=0, j=0; i<len; ++i) {
-    if (list[i].className == classname) {
-      found[j++] = list[i];
+    var classes = list[i].className.split(' ');
+    for (var k=0; k<classes.length; ++k) {
+      if (classes[k] == classname) {
+        found[j++] = list[i];
+        break;
+      }
     }
   }
   delete list;
@@ -104,10 +103,14 @@ function high(e,on) {
       list[i].style.background = col;
       if (tip) list[i].title += tip;
     }
-    // use class name to highlight corresponding span elements
-    list = GetSpansByClass(targ.name);
-    for (var i=0; i<list.length; ++i) {
-      list[i].style.background = col;
+    // use class name to highlight span elements if necessary
+    for (var i=0; i<mspan.length; ++i) {
+      if (mspan[i] != targ.name) continue;
+      list = GetElementsByClass(targ.name, 'span');
+      for (var j=0; j<list.length; ++j) {
+        list[j].style.background = col;
+      }
+      break;
     }
   }
 }
@@ -116,9 +119,9 @@ _END_PART_2_
 my $htmlHeader3 = q[
 // End --->
 </script></head>
-<body bgcolor='#ffffff'><noscript><b id='V'>--&gt;
+<body><noscript><b class=V>--&gt;
 Enable JavaScript for active highlighting and information tool tips!
-</b></noscript><table class='dump' cellspacing=0 cellpadding=2>
+</b></noscript><table class=dump cellspacing=0 cellpadding=2>
 <tr><td valign='top'><pre>];
 
 my $preMouse = q(<pre onmouseover="high(event,1)" onmouseout="high(event,0)">);
@@ -138,10 +141,10 @@ sub new
 # Add information to dump
 # Inputs: 0) HTML dump hash ref, 1) absolute offset in file, 2) data size,
 #         3) comment string, 4) tool tip (or SAME to use previous tip),
-#         5) bit flags (see below)
+#         5) bit flags (see below), 6) true to use same tooltip as last call
 # Bits: 0x01 - print at start of line
 #       0x02 - print red address
-#       0x04 - make red background
+#       0x04 - maker notes data ('M'-class span)
 #       0x08 - limit block length
 # Notes: Block will be shown in 'unused' color if comment string begins with '['
 sub Add($$$$;$$)
@@ -152,6 +155,12 @@ sub Add($$$$;$$)
     if ($tip and $tip eq 'SAME') {
         $tip = '';
     } else {
+        $tip = defined $tip ? '\n' . $tip : '';
+        my $m = $msg;
+        $m =~ s/<.*?>//g;       # remove html format codes
+        $tip = "$m$tip";        # add msg as first line in tooltip
+        # add size if not already done
+        $tip .= "\\n($size bytes)" unless $tip =~ /\\nSize:/;
         ++$self->{TipNum};
     }
     push @{$$block{$start}}, [ $size, $msg, $tip, $flag, $self->{TipNum} ];
@@ -162,12 +171,13 @@ sub Add($$$$;$$)
 # Inputs: 0) Dump information hash reference, 1) source file RAF reference,
 #         2) data pointer, 3) data position, 4) output file or scalar reference,
 #         5) limit level (1-3), 6) title
-# Returns: non-zero if useful output was generated
+# Returns: non-zero if useful output was generated,
+#          or -1 on error loading data and ERROR is set to offending data name
 sub Print($$;$$$$$)
 {
     local $_;
     my ($self, $raf, $dataPt, $dataPos, $outfile, $level, $title) = @_;
-    my ($i, $buff, $rtnVal);
+    my ($i, $buff, $rtnVal, $limit, $err);
     my $block = $$self{Block};
     $dataPos = 0 unless $dataPos;
     $outfile = \*STDOUT unless ref $outfile;
@@ -175,21 +185,23 @@ sub Print($$;$$$$$)
     $level or $level = 0;
     my $tell = $raf->Tell();
     my @starts = sort { $a <=> $b } keys %$block;
-    my $pos = $starts[0] || 0;
+    my $pos = 0;
     my $dataEnd = $dataPos + ($dataPt ? length($$dataPt) : 0);
     # initialize member variables
     $$self{Open} = [];
     $$self{Closed} = [];
     $$self{TipList} = [];
+    $$self{MSpanList} = [];
     $$self{Cols} = [ '', '', '', '' ];  # text columns
     # set dump size limits (limits are 4x smaller if bit 0x08 set in flags)
     if ($level <= 1) {
-        $$self{Limit} = 1024;
+        $limit = 1024;
     } elsif ($level <= 2) {
-        $$self{Limit} = 16384;
+        $limit = 16384;
     } else {
-        delete $$self{Limit};   # no limit
+        $limit = 256 * 1024 * 1024; # never dump bigger than 256 MB
     }
+    $$self{Limit} = $limit;
     # pre-initialize open/closed hashes for all columns
     for ($i=0; $i<4; ++$i) {
         $self->{Open}->[$i] = { ID => [ ], Element => { } };
@@ -197,54 +209,84 @@ sub Print($$;$$$$$)
     }
     $bkgStart = $bkgEnd = 0;
     $bkgSpan = '';
-    my $index = 0;  # numerical index corresponding to 'AA'
-    my @names;
+    my $index = 0;  # initialize tooltip index
+    my (@names, $wasUnused);
     for ($i=0; $i<@starts; ++$i) {
         my $start = $starts[$i];
-        my $bytes = $start - $pos;
-        if ($bytes > 0) {
-            if ($pos >= $dataPos and $pos + $bytes <= $dataEnd) {
-                $buff = substr($$dataPt, $pos-$dataPos, $bytes);
-            } else {
-                $raf->Seek($pos, 0);
-                $raf->Read($buff, $bytes);
-            }
-            my $str = ($bytes > 1) ? "unused $bytes bytes" : 'pad byte';
-            $self->DumpTable($pos-$dataPos, \$buff, "[$str]", "t$index", 0x108);
-            ++$index;
-            $pos = $start;  # dumped unused data up to the start of this block
+        my $parmList = $$block{$start};
+        my $len = $start - $pos;
+        if ($len > 0 and not $wasUnused) {
+            # we have an unused bytes before this data block
+            --$i;           # dump the data block next time around
+            $start = $pos;  # dump the unused bytes now
+            my $str = ($len > 1) ? "unused $len bytes" : 'pad byte';
+            $parmList = [ [ $len, "[$str]", undef, 0x108 ] ];
+            $wasUnused = 1; # avoid re-dumping unused bytes if we get a read error
+        } else {
+            undef $wasUnused;
         }
         my $parms;
-        my $parmList = $$block{$start};
         foreach $parms (@$parmList) {
             my ($len, $msg, $tip, $flag, $tipNum) = @$parms;
             next unless $len > 0;
             $flag = 0 unless defined $flag;
-            # generate same name for all blocks starting with the same message word
-            my $name = $names[$tipNum];
+            # generate same name for all blocks indexed by this tooltip
+            my $name;
+            $name = $names[$tipNum] if defined $tipNum;
             my $idx = $index;
             if ($name) {
                 # get index from existing ID
                 $idx = substr($name, 1);
             } else {
-                $name = $names[$tipNum] = "t$index";
+                $name = "t$index";
+                $names[$tipNum] = $name if defined $tipNum;
                 ++$index;
             }
             if ($flag == 4) {
                 $bkgStart = $start - $dataPos;
                 $bkgEnd = $bkgStart + $len;
-                # strictly speaking, 'span' doesn't permit 'name', so use 'class' instead
-                $bkgSpan = "<span id='M' class='$name'>";
+                $bkgSpan = "<span class='$name M'>";
+                push @{$self->{MSpanList}}, $name;
                 next;
             }
-            $tip and $self->{TipList}->[$idx] = $tip;
-            my $end = $start + $len;
-            if ($start >= $dataPos and $end <= $dataEnd) {
-                $buff = substr($$dataPt, $start-$dataPos, $len);
-            } else {
-                $raf->Seek($start, 0);
-                $raf->Read($buff, $len);
+            # loop until we read the value properly
+            my ($end, $try);
+            for ($try=0; $try<2; ++$try) {
+                $end = $start + $len;
+                # only load as much of the block as we are going to dump
+                my $size = ($len > $limit) ? $limit / 2 : $len;
+                if ($start >= $dataPos and $end <= $dataEnd) {
+                    $buff = substr($$dataPt, $start-$dataPos, $size);
+                    if ($len != $size) {
+                        $buff .= substr($$dataPt, $start-$dataPos+$len-$size, $size);
+                    }
+                } else {
+                    $buff = '';
+                    if ($raf->Seek($start, 0) and $raf->Read($buff, $size) == $size) {
+                        # read end of block
+                        if ($len != $size) {
+                            my $buf2 = '';
+                            unless ($raf->Seek($start+$len-$size, 0) and
+                                    $raf->Read($buf2, $size) == $size)
+                            {
+                                $err = $msg;
+                                # reset $len to the actual length of available data
+                                $raf->Seek(0, 2);
+                                $len = $raf->Tell() - $start;
+                                $tip .= "\\nError: Only $len bytes available!" if $tip;
+                                next;
+                            }
+                            $buff .= $buf2;
+                            undef $buf2;
+                        }
+                    } else {
+                        $err = $msg;
+                    }
+                }
+                last;
             }
+            $tip and $self->{TipList}->[$idx] = $tip;
+            next unless length $buff;
             # set flag to continue this line if next block is contiguous
             if ($i+1 < @starts and $parms eq $$parmList[-1] and
                 ($end == $starts[$i+1] or ($end < $starts[$i+1] and $end >= $pos)))
@@ -253,7 +295,8 @@ sub Print($$;$$$$$)
                 $flag |= 0x100 unless $flag & 0x01 or $nextFlag & 0x01;
             }
             $self->DumpTable($start-$dataPos, \$buff, $msg, $name,
-                             $flag, $pos-$dataPos);
+                             $flag, $len, $pos-$dataPos);
+            undef $buff;
             $pos = $end if $pos < $end;
         }
     }
@@ -264,6 +307,10 @@ sub Print($$;$$$$$)
     Write($outfile, $htmlHeader1, $title);
     if ($self->{Cols}->[0]) {
         Write($outfile, $htmlHeader2);
+        my $mspan = \@{$$self{MSpanList}};
+        for ($i=0; $i<@$mspan; ++$i) {
+            Write($outfile, qq(mspan[$i] = "$$mspan[$i]";\n));
+        }
         my $tips = \@{$$self{TipList}};
         for ($i=0; $i<@$tips; ++$i) {
             Write($outfile, qq(t[$i] = "$$tips[$i]";\n)) if defined $$tips[$i];
@@ -272,7 +319,7 @@ sub Print($$;$$$$$)
         Write($outfile, $htmlHeader3, $self->{Cols}->[0]);
         Write($outfile, '</pre></td><td valign="top">',
                         $preMouse, $self->{Cols}->[1]);
-        Write($outfile, '</pre></td><td class="c2" valign="top">',
+        Write($outfile, '</pre></td><td class=c2 valign="top">',
                         $preMouse, $self->{Cols}->[2]);
         Write($outfile, '</pre></td><td valign="top">',
                         $preMouse, $self->{Cols}->[3]);
@@ -281,11 +328,17 @@ sub Print($$;$$$$$)
     } else {
         Write($outfile, "$title</title></head><body>\n",
                         "No EXIF or TIFF information found in image\n");
+        $rtnVal = 0;
     }
     Write($outfile, "</body></html>\n");
     for ($i=0; $i<4; ++$i) {
         $self->{Cols}->[$i] = '';   # free memory
     }
+    if ($err) {
+        $err =~ tr/()//d;
+        $$self{ERROR} = $err;
+        return -1;
+    }
     return $rtnVal;
 }
 
@@ -370,16 +423,18 @@ sub Open($$$;@)
 #------------------------------------------------------------------------------
 # Dump a block of data in HTML table form
 # Inputs: 0) HtmlDump object ref, 1) data position, 2) block pointer,
-#         3) message, 4) object name, 5) flag, 6) data end position
-sub DumpTable($$$;$$$$)
+#         3) message, 4) object name, 5) flag, 6) full block length (actual
+#         data may be shorter), 7) data end position
+sub DumpTable($$$;$$$$$)
 {
-    my ($self, $pos, $blockPt, $msg, $name, $flag, $endPos) = @_;
-    my $len = length $$blockPt;
+    my ($self, $pos, $blockPt, $msg, $name, $flag, $len, $endPos) = @_;
+    $len = length $$blockPt unless defined $len;
     $endPos = 0 unless $endPos;
     my ($f0, $dblRef, $id);
+    my $skipped = 0;
     if (($endPos and $pos < $endPos) or $flag & 0x02) {
         # display double-reference addresses in red
-        $f0 = "<span id='V'>";
+        $f0 = "<span class=V>";
         $dblRef = 1 if $endPos and $pos < $endPos;
     } else {
         $f0 = '';
@@ -399,7 +454,7 @@ sub DumpTable($$$;$$$$)
             }
             ++$id unless $dblRef;
         }
-        $name = qq{<a name='$name' id='$id'>};
+        $name = qq{<a name=$name class=$id>};
         $msg and $msg = "$name$msg</a>";
     } else {
         $name = '';
@@ -430,7 +485,7 @@ sub DumpTable($$$;$$$$)
     for (;;) {
         $self->Open('bkg', ($p>=$bkgStart and $p<$bkgEnd) ? $bkgSpan : '', 1, 2);
         $self->Open('a', $name, 1, 2);
-        my $ch = substr($$blockPt,$p-$pos,1);
+        my $ch = substr($$blockPt,$p-$pos-$skipped,1);
         $c[1] .= sprintf("%.2x", ord($ch));
         # make the character HTML-friendly
         $ch =~ tr/\x00-\x1f\x7f-\xff/./;
@@ -449,7 +504,7 @@ sub DumpTable($$$;$$$$)
         if ($dblRef and $p >= $endPos) {
             $dblRef = 0;
             ++$id;
-            $name =~ s/id='.'/id='$id'/;
+            $name =~ s/class=\w\b/class=$id/;
             $f0 = '';
             $self->Open('fgd', $f0, 0);
         }
@@ -460,6 +515,9 @@ sub DumpTable($$$;$$$$)
         if ($cols < 16) {
             $c[1] .= ($cols == 8 ? '  ' : ' ');
             next;
+        } elsif ($flag & 0x01 and $cols < $len) {
+            $c[1] .= ' ';
+            next;   # put it all on one line
         }
         unless ($$self{Msg}) {
             $c[3] .= $msg;
@@ -482,6 +540,7 @@ sub DumpTable($$$;$$$$)
                     $c[2] .= "     [snip]     \n";
                     $c[3] .= "\n";
                     $p += $n;
+                    $skipped += $len - length $$blockPt;
                 }
             }
         }
@@ -508,6 +567,97 @@ sub DumpTable($$$;$$$$)
     delete $$self{TmpCols};
 }
 
+#------------------------------------------------------------------------------
+# Finish dumping of TIFF image data
+# Inputs: 0) HtmlDump object ref, 1) ExifTool object ref, 2) length of file
+# (this really belongs in Image::ExifTool::Exif, but is placed here so it
+#  is only compiled when needed)
+sub FinishTiffDump($$$)
+{
+    my ($self, $exifTool, $size) = @_;
+    my ($tag, $key, $start, $blockInfo, $i);
+
+    # list of all indirectly referenced TIFF data tags
+    my %offsetPair = (
+        StripOffsets      => 'StripByteCounts',
+        TileOffsets       => 'TileByteCounts',
+        FreeOffsets       => 'FreeByteCounts',
+        ThumbnailOffset   => 'ThumbnailLength',
+        PreviewImageStart => 'PreviewImageLength',
+        JpgFromRawStart   => 'JpgFromRawLength',
+        OtherImageStart   => 'OtherImageLength',
+        ImageOffset       => 'ImageByteCount',
+        AlphaOffset       => 'AlphaByteCount',
+    );
+
+    # add TIFF data to html dump
+    foreach $tag (keys %offsetPair) {
+        my $info = $exifTool->GetInfo($tag);
+        next unless %$info;
+        foreach $key (%$info) {
+            my $name = Image::ExifTool::GetTagName($key);
+            my $grp1 = $exifTool->GetGroup($key, 1);
+            my $info2 = $exifTool->GetInfo($offsetPair{$tag}, { Group1 => $grp1 });
+            next unless %$info2;
+            my ($key2) = keys %$info2;
+            my $offsets = $$info{$key};
+            my $byteCounts = $$info2{$key2};
+            # (long lists may be SCALAR references)
+            my @offsets = split ' ', (ref $offsets ? $$offsets : $offsets);
+            my @byteCounts = split ' ', (ref $byteCounts ? $$byteCounts : $byteCounts);
+            my $num = scalar @offsets;
+            my $li = 0;
+            my $padBytes = 0;
+            for ($i=0; @offsets and @byteCounts; ++$i) {
+                my $offset = shift @offsets;
+                my $byteCount = shift @byteCounts;
+                my $end = $offset + $byteCount;
+                if (@offsets and @byteCounts) {
+                    # show data as contiguous if only normal pad bytes between blocks
+                    if ($end & 0x01 and $end + 1 == $offsets[0]) {
+                        $end += 1;
+                        ++$padBytes;    # count them
+                    }
+                    if ($end == $offsets[0]) {
+                        # combine these two blocks
+                        $byteCounts[0] += $offsets[0] - $offset;
+                        $offsets[0] = $offset;
+                        next;
+                    }
+                }
+                my $msg = $exifTool->GetGroup($key, 1) . ':' . $tag;
+                $msg =~ s/(Offsets?|Start)$/ /;
+                if ($num > 1) {
+                    $msg .= "$li-" if $li != $i;
+                    $msg .= "$i ";
+                    $li = $i + 1;
+                }
+                $msg .= "data";
+                my $tip = "Size: $byteCount bytes";
+                $tip .= ", incl. $padBytes pad bytes" if $padBytes;
+                $self->Add($offset, $byteCount, "($msg)", $tip, 0x08);
+            }
+        }
+    }
+    # find offset of last dumped information, and dump any unknown trailer
+    my $last = 0;
+    my $block = $$self{Block};
+    foreach $start (keys %$block) {
+        foreach $blockInfo (@{$$block{$start}}) {
+            my $end = $start + $$blockInfo[0];
+            $last = $end if $last < $end;
+        }
+    }
+    my $diff = $size - $last;
+    if ($diff > 0 and ($last or $exifTool->Options('Unknown'))) {
+        if ($diff > 1 or $size & 0x01) {
+            $self->Add($last, $diff, "[unknown data]", "Size: $diff bytes", 0x08);
+        } else {
+            $self->Add($last, $diff, "[trailing pad byte]", undef, 0x08);
+        }
+    }
+}
+
 #------------------------------------------------------------------------------
 # utility routine to write to file or memory
 # Inputs: 0) file or scalar reference, 1-N) list of stuff to write
@@ -559,7 +709,7 @@ display linefeeds in the tool tips.
 
 =head1 AUTHOR
 
-Copyright 2003-2006, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2008, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
