@@ -28,7 +28,7 @@ use strict;
 use vars qw($VERSION $AUTOLOAD);
 use Image::ExifTool qw(:DataAccess :Utils);
 
-$VERSION = '1.13';
+$VERSION = '1.17';
 
 sub ProcessPNG_tEXt($$$);
 sub ProcessPNG_iTXt($$$);
@@ -161,7 +161,7 @@ $Image::ExifTool::PNG::colorType = -1;
             return pack('nC5', @a);
         },
         PrintConv => '$self->ConvertDateTime($val)',
-        PrintConvInv => '$val',
+        PrintConvInv => '$self->InverseDateTime($val)',
     },
     tRNS => {
         Name => 'Transparency',
@@ -271,11 +271,12 @@ listed below are the only ones that can be written (unless new user-defined
 tags are added via the configuration file), however ExifTool will extract
 any other TextualData tags that are found.
 
-The information for the TextualData tags may be stored as tEXt, zTXt or iTXt
-chunks in the PNG image.  ExifTool will read and edit tags in their original
-form, but tEXt chunks are written by default when creating new tags.
-Compressed zTXt chunks are written only if Compress::Zlib is available, and
-only for profile information or when the -z (Compress) option is specified.
+Data for the TextualData tags may be stored as tEXt, zTXt or iTXt chunks in
+the PNG image.  ExifTool will read and edit tags in their original form, but
+new string tags are created as uncompressed tEXt by default, or as
+compressed zTXt if the -z (Compress) option is used and Compress::Zlib is
+available. Raw profile information is always created as compressed zTXt if
+Compress::Zlib is available.
 
 Some of the tags below are not registered as part of the PNG specification,
 but are included here because they are generated by other software such as
@@ -472,7 +473,7 @@ sub FoundPNG($$$$;$$)
                 return 1 unless $exifTool->{EDIT_DIRS}->{$tagName};
                 $$outBuff = $exifTool->WriteDirectory(\%subdirInfo, $subTable);
                 # if this was an XMP directory, we must make it read-only
-                $tagName eq 'XMP' and Image::ExifTool::XMP::ValidateXMP($outBuff,'r');
+                $tagName eq 'XMP' and Image::ExifTool::XMP::ValidateXMP(\$outBuff,'r');
                 delete $exifTool->{ADD_DIRS}->{$tagName};
             } else {
                 $processed = $exifTool->ProcessDirectory(\%subdirInfo, $subTable, $processProc);
@@ -486,7 +487,7 @@ sub FoundPNG($$$$;$$)
             {
                 # write new value for this tag if necessary
                 my ($isOverwriting, $newVal);
-                if ($exifTool->{DEL_GROUP} and $exifTool->{DEL_GROUP}->{PNG}) {
+                if ($exifTool->{DEL_GROUP}->{PNG}) {
                     # remove this tag now, but keep in ADD_PNG list to add back later
                     $isOverwriting = 1;
                 } else {
@@ -548,7 +549,7 @@ sub FoundPNG($$$$;$$)
         ($name = $tag) =~ s/\s+(.)/\u$1/g;   # remove white space from tag name
         $tagInfo = { Name => $name };
         # make unknown profiles binary data type
-        $$tagInfo{ValueConv} = '\$val' if $tag =~ /^Raw profile type /;
+        $$tagInfo{Binary} = 1 if $tag =~ /^Raw profile type /;
         Image::ExifTool::AddTagToTable($tagTablePtr, $tag, $tagInfo);
     }
 #
@@ -781,7 +782,8 @@ sub ProcessPNG($$)
         $exifTool->{ADD_PNG} = $exifTool->GetNewTagInfoHash(
             \%Image::ExifTool::PNG::Main,
             \%Image::ExifTool::PNG::TextualData);
-        $exifTool->InitWriteDirs('JPEG');   # use same directories as JPEG
+        # initialize with same directories as JPEG, but PNG tags take priority
+        $exifTool->InitWriteDirs('JPEG','PNG');
     }
     my ($fileType, $hdrChunk, $endChunk) = @{$pngLookup{$sig}};
     $exifTool->SetFileType($fileType);  # set the FileType tag
@@ -845,6 +847,8 @@ sub ProcessPNG($$)
         unless ($foundHdr) {
             if ($chunk eq $hdrChunk) {
                 $foundHdr = 1;
+            } elsif ($hdrChunk eq 'IHDR' and $chunk eq 'CgBI') {
+                $exifTool->Warn('Non-standard PNG image (Apple iPhone format)');
             } else {
                 $exifTool->Warn("$fileType image did not start with $hdrChunk");
                 last;
@@ -861,7 +865,7 @@ sub ProcessPNG($$)
             }
             print $out "$fileType $chunk ($len bytes):\n";
             if ($verbose > 2) {
-                my %dumpParms = ( Out => $out );
+                my %dumpParms = ( Out => $out, Addr => $raf->Tell() - $len - 4 );
                 $dumpParms{MaxLen} = 96 if $verbose <= 4;
                 Image::ExifTool::HexDump(\$dbuf, undef, %dumpParms);
             }
@@ -911,7 +915,7 @@ and JNG (JPEG Network Graphics) images.
 
 =head1 AUTHOR
 
-Copyright 2003-2006, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2008, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
